%ising
% Ising model of Ferromagnetism
% LAST UPDATED by Andy French October 2019
%
% References:
% https://uk.mathworks.com/matlabcentral/...
% fileexchange/62194-ising-model-and-metropolis-algorithm

function ising

%Ising model inputs
input_str = 'default full model run';
input_str = 'Single repeat and temp N=10 with animation';
input_str = 'Single repeat and temp N=100 with animation';
input_str = 'Single repeat and temp N=500 with animation';
input_str = 'quick test';
input_str = 'fricke N=100';

input_str = 'default full model run';
[ show_animation, N,I,R,x,kT,Tc,J,F,B,m,fricke ] = ising_inputs( input_str );

%Fontsize
fsize = 18;

%

%Run metropolis algorithm for range of temperatures
disp(' ');
disp(' ISING MODEL OF FERROMAGNETISM by Andy French Oct 2019')
disp(' ');

%Start clock
t0 = tic;

%Initialize stores of mean spin, mean absolute spin, mean energy per spin,
%heat capacity, and standard deviations of these.
Nx = numel(x);
sm = zeros(1,R*Nx) ; xx = zeros(1,R*Nx);
abs_sm = zeros(1,Nx); Em = zeros(1,Nx); C = zeros(1,Nx);
abs_sm_sd = zeros(1,Nx); Em_sd = zeros(1,Nx); C_sd = zeros(1,Nx);

%Run R repeats of Metropolis algorithm for each x = T/Tc value
i = 1;
for n=1:Nx
    Emr = zeros(1,R); Cr = zeros(1,R); abs_smr = zeros(1,R);
    for r=1:R
        %Display message to command window
        disp([' Ising model run: ',num2str(i),'/',num2str( R*length(x) ),...
            ' T/Tc = ',num2str( x(n) ),'. Repeat ',...
            num2str(r),' of ',num2str(R)]);
        
        %Run I*N*N iterations of Meropolis algorithm
        [ s,smi,Emi,Ci ] = metropolis( N, I, kT(n), Tc*x(n), Tc, J, F,...
            show_animation, fsize, fricke );
        
        %Update output stores
        xx = [xx,x(n)]; sm = [ sm, smi ];
        Emr(r) = Emi; Cr(r) = Ci; abs_smr(r) = abs(smi);
        
        %Update loop counter
        i = i + 1;
    end
    
    %Store averages of absolute spin, mean energy/spin, heat capacity
    %and standard deviations of these
    abs_sm(n) = mean( abs_smr ); abs_sm_sd(n) = std( abs_smr );
    Em(n) = mean( Emr); Em_sd(n) = std( Emr );
    C(n) = mean( Cr); C_sd(n) = std( Cr );
end

%Save data to a .mat file and display completion message
save ising
disp(' ');
disp([' Simulation complete in ',num2str(toc(t0)),'s.']);
disp(' ');

%Plot average spin vs T/Tc
figure('color',[1 1 1],'name','Ising model');
plot( xx,  sm, '.' ); hold on;
patch( [x,fliplr(x)],[ abs_sm + abs_sm_sd, fliplr( abs_sm - abs_sm_sd )],'g','facealpha',0.5 );
plot( x, abs_sm, 'r-','linewidth',2 ); axis tight
try xlim( [min(x),max(x)] ); end
xlabel('T/T_c','fontsize',fsize); ylabel('Average spin','fontsize',fsize);
title(['Average spin. J=',num2str(J,2),'eV. T_c=',num2str(Tc),' K'],'fontsize',fsize);
grid on; box on;
set(gca,'fontsize',fsize);
print(gcf,['Ising spin vs T N=',num2str(N),'.png'],'-dpng','-r300');
close(gcf);

%Plot average spin energy /eV vs T/Tc
figure('color',[1 1 1],'name','Ising model'); hold on;
patch( [x,fliplr(x)],[ Em + Em_sd, fliplr( Em - Em_sd )]/J,'g','facealpha',0.5 );
plot( x, Em/J, 'r-','linewidth',2 ); axis tight
try xlim( [min(x),max(x)] ); end
xlabel('T/T_c','fontsize',fsize); ylabel('Spin energy/coupling energy J','fontsize',fsize);
title(['Spin energy/J. J=',num2str(J,2),'eV. T_c=',num2str(Tc),' K'],'fontsize',fsize);
grid on;  box on;
set(gca,'fontsize',fsize);
print(gcf,['Spin energy vs T N=',num2str(N),'.png'],'-dpng','-r300');
close(gcf);

%Plot heat capacity /eV per K vs T/Tc
figure('color',[1 1 1],'name','Ising model'); hold on;
patch( [x,fliplr(x)],[ C + C_sd, fliplr( C - C_sd )],'g','facealpha',0.5 );
plot( x, C, 'r-','linewidth',2 ); axis tight
try xlim( [min(x),max(x)] ); end
xlabel('T/T_c','fontsize',fsize); ylabel('Heat capacity / eV/K','fontsize',fsize);
title(['Heat capacity. J=',num2str(J,2),'eV. T_c=',num2str(Tc),' K'],'fontsize',fsize);
grid on; box on;
set(gca,'fontsize',fsize);
print(gcf,['Heat capacity vs T N=',num2str(N),'.png'],'-dpng','-r300');
close(gcf);

%%

%Ising model inputs
function [ show_animation, N,I,R,x,kT,Tc,J,F,B,m,fricke ] = ising_inputs( input_choice )
if strcmp( input_choice, 'default full model run' )
    %Show Metropolis algorithm animation (creates a .avi movie, and also plots a
    %graph of average spin vs iteration number)
    show_animation = 0;
    
    %Number of magnetic dipoles (NxN grid)
    N = 10;
    
    %Iterations of Metropolis algorithm (as a multiple of N^2)
    I = 100;
    
    %Repeats of Metropolis algorithm per temperature
    R = 2000;
    
    %Temperature range where x = T/Tc
    xmin = 0.2; xmax = 0.4;
    Nx = 4;
    
    %Curie temperature /K
    % https://en.wikipedia.org/wiki/Curie_temperature
    % Iron 1043, Cobalt 1400, Nickel 627
    Tc = 1043;
    
    %Applied field strength /T
    B = 0.0;
    
    %Magnetic dipole moment /JT^-1
    m = 9284.764e-27; %Single electron
    
    %Use Fricke vectorized method in Metropolis algorithm. This makes things
    %much faster, but you will need more iterations I.
    fricke = 0;
elseif strcmp( input_choice, 'Single repeat and temp N=100 with animation' )
    show_animation = 1;
    N = 100;
    I = 50;
    R = 1;
    xmin = 0.5; xmax = 0.5;
    Nx = 1;
    Tc = 1043;
    B = 0.0;
    m = 9284.764e-27;
    fricke = 0;
elseif strcmp( input_choice, 'Single repeat and temp N=10 with animation' )
    show_animation = 1;
    N = 10;
    I = 25;
    R = 1;
    xmin = 0.5; xmax = 0.5;
    Nx = 1;
    Tc = 1043;
    B = 0.0;
    m = 9284.764e-27;
    fricke = 0;
elseif strcmp( input_choice, 'quick test' )
    show_animation = 0;
    N = 10;
    I = 25;
    R = 10;
    xmin = 0.2; xmax = 1.6;
    Nx = 10;
    Tc = 1043;
    B = 0.0;
    m = 9284.764e-27;
    fricke = 1;
elseif strcmp( input_choice, 'fricke N=100' )
    show_animation = 0;
    N = 100;
    I = 500;
    R = 200;
    xmin = 0.2; xmax = 2.0;
    Nx = 100;
    Tc = 1043;
    B = 0.0;
    m = 9284.764e-27;
    fricke = 1;
elseif strcmp( input_choice, 'Single repeat and temp N=500 with animation' )
    show_animation = 1;
    N = 500;
    I = 2000;
    R = 1;
    xmin = 0.5; xmax = 0.5;
    Nx = 1;
    Tc = 1043;
    B = 0.0;
    m = 9284.764e-27;
    fricke = 1;
else
    %As per 'Single repeat and temp with animation'
    show_animation = 1;
    N = 100;
    I = 50;
    R = 1;
    xmin = 0.5; xmax = 0.5;
    Nx = 1;
    Tc = 1043;
    B = 0.0;
    m = 9284.764e-27;
    fricke = 1;
end

%Charge on the electron /C
qe = 1.6e-19;

%Boltzmann's constant (JK^-1);
kB = 1.38e-23;

%Magnetic dipole nearest-neighbour coupling energy /eV
%from Curie temperature (for 2D rectangular grid Ising model)
%using Lars Onsager's classic result
J = Tc * log( 1+sqrt(2) )*kB / (2*qe);

%Determine array of x = T/Tc
x = linspace( xmin, xmax, Nx );

%Determine kB*T in eV
kT = Tc*x*kB/qe;

%Determine applied magnetic field energy /eV
F = m*B/qe;

%%

%metropolis
% Determine average spin state using Metropolis algorithm
function [ s,sm,Em,C ] = metropolis( N, I, kT, T, Tc, J, F, show_animation, fsize, fricke )

%Initialize spin states (+1,-1) of grid
s = sign( 0.5 - rand(N,N) );

%Determine mean spin, mean spin energy /eV and heat capacity /eVK^-1
[ sm, Em, C ] = mean_spin_calcs( s, J, kT, T );
sm_store = [sm,zeros(1,I-1)];

if show_animation==1
    %Initialize plot
    figure('name','ising','color',[1 1 1],'units','normalized','position',[0,0,1,1] );
    p = pcolor(s); axis off; axis equal;
    tit = title({['iteration = ',num2str(1),'/',num2str(I)],...
        ['Mean spin=',num2str(sm),', T/Tc=',num2str(T/Tc)]},'fontsize',fsize );
    shading flat; box on;
    colormap( [0,0,0;1,1,1] );
    
    %Print first frame
    print( gcf, ['Ising FIRST FRAME ','N=',num2str(N),...
        ', x=',strrep( num2str(T/Tc),'.','p'),'.png'],'-dpng','-r300' );
    
    %Open videowriter object
    vidObj = VideoWriter( ['Ising animation ','N=',num2str(N),...
        ', x=',strrep( num2str(T/Tc),'.','p'),'.avi'] );
    open(vidObj);
end

%Run Ising model via the Metropolis algorithm
% Choose a random magnetic dipole to be flipped and work out total energy
% change. If energy change dE is negative, flip. If energy change is
% positive, flip if exp( -dE/(kB*T) ) > rand
for i=1:I
    if fricke==0
        %Original Metropolis method, making one change at a time.
        for j=1:N*N
            %Get rows, cols of randomly selected magnetic dipole
            [row,col] = ind2sub( [N,N], randi(N*N) );
            
            %Determine indices of four neighbouring spins
            rows = [row,row+1,row,row-1]; cols = [col-1,col,col+1,col];
            
            %Implement circular boundary condition if selected spin is at the edge
            %of the array
            rows(rows>N) = 1; rows(rows<1) = N; cols(cols>N) = 1; cols(cols<1) = N;
            neighbours = ones(1,4);
            for n=1:4; neighbours(n) = s( rows(n),cols(n) ); end
            
            %Decide whether to change the sign of spin s(n)
            dE = 2*J * sum(neighbours) * s(row,col) + 2*F*s(row,col) ; %in eV
            if ( dE<0 ) || ( exp(-dE/kT) >= rand() )
                s(row,col) = -s(row,col);
            end
        end
    else
        %Fricke's much faster vectorized version of the Metropolis algorithm
        %(Tobin Fricke 2006)
        
        % Calculate the number of neighbours of each cell
        neighbours = circshift( s, [ 0 1]) + ...
            circshift( s, [ 0 -1]) + ...
            circshift( s, [ 1 0]) + ...
            circshift( s, [-1 0]);
        
        % Calculate the change in energy /eV of flipping a spin
        dE = 2*J *( s .* neighbours) + 2*F*s ;
        
        % Decide which transitions will occur
        transitions = ( rand(N) < exp( -dE/kT ) ).*(rand(N) < 0.1) * -2 + 1;
        
        % Flip spins
        s = s.*transitions;
    end
    
    %Determine mean spin, mean spin energy /eV and heat capacity /eVK^-1
    [ sm, Em, C ] = mean_spin_calcs( s, J, kT, T );
    sm_store(i)  = sm;
    
    %Update animation, if show_animation==1
    if show_animation ==1
        
        %Update figure
        set( p,'cdata',s );
        set( tit, 'string',{ ['iteration = ',num2str(i),'/',num2str(I)],...
            ['Mean spin=',num2str(sm),', T/Tc=',num2str(T/Tc)] },'fontsize',fsize );
        drawnow;
        
        %Add frame to .avi video
        writeVideo( vidObj, getframe );
    end
end
if show_animation==1
    %Save last frame
    print( gcf, ['Ising LAST FRAME ','N=',num2str(N),...
        ', x=',strrep( num2str(T/Tc),'.','p'),'.png'],'-dpng','-r300' );
    close(gcf);
    
    %Plot progress of average spin vs iteration
    figure; plot( 1:I,sm_store );
    xlabel('Iteration/N^2','fontsize',18);
    ylabel('Average spin','fontsize',18);
    title(['Mean spin vs iteration. N=',num2str(N),...
        ', T/Tc=',num2str(T/Tc)],'fontsize',fsize);
    set(gca,'fontsize',fsize); grid on;
    print( gcf, ['smean vs iter ','N=',num2str(N),...
        ', x=',strrep( num2str(T/Tc),'.','p'),'.png'],'-dpng','-r300' );
    close(gcf);
    
    %Close videowriter object
    close(vidObj);
end

%%

%Mean energy per spin E /eV given N*N spin array (of -1,1) and coupling
%energy J/eV. Variance of E is also computed, which is then used to
%determine the heat capacity C in eV/K.
function [ sm, Em, C ] = mean_spin_calcs( s, J, kT, T )

%Calculate average spin
sm = mean( s(:) );

%Determine spins of nearest neighbour interactions. A neat implementation
%from https://uk.mathworks.com/matlabcentral/fileexchange/...
%62194-ising-model-and-metropolis-algorithm
sumOfNeighbors = ...
    circshift(s, [ 0  1]) + circshift(s, [ 0 -1]) ...
    + circshift(s, [ 1  0]) + circshift(s, [-1  0]);

%Matrix of spin coupling energies
E = - J * s .* sumOfNeighbors;

%Average energy per spin. Note 0.5 multiplier to avoid double counting
% spin coupling interactions
Em  = 0.5 * mean( E(:) );

%Compute variance of mean energy (which can be used to compute
%specific heat capacity C = Var(E)/(kB*T^2) in eV/K
C = 0.5 * var( E(:) )/( kT*T );

%End of code




