%ising
% Ising model of Ferromagnetism
% LAST UPDATED by Andy French October 2019
%
% References:
% https://uk.mathworks.com/matlabcentral/...
% fileexchange/62194-ising-model-and-metropolis-algorithm

function ising

%% Ising model inputs %%

%Model runs to generate spin etc vs T/Tc
input_str = 'N=10, I=25, R=10, x=0.0 to 2.0, Nx=10, F=0';
input_str = 'N=10, I=2000, R=100, x=0.0 to 2.0, Nx=100, F=0';
input_str = 'N=10, I=100, R=10, x=0.0 to 0.4, Nx=3 check convergence, F=0';
input_str = 'Fricke N=100, I=1000, R=100, x=0.0 to 2.0, Nx=100, F=0';

%Animations (single repeat, single temp)
input_str = 'N=100, I=50, R=1, x=0.5 with animation, F=0';
input_str = 'Fricke N=500, I=1000, R=1, x=0.5 with animation, F=0';
input_str = 'Fricke N=100, I=1000, R=1, x=0.5 with animation, F=0';

%Get model inputs
input_str = 'N=10, I=2000, R=100, x=0.0 to 2.0, Nx=100, F=0';
[ show_animation,show_s_vs_iter, N,I,R,x,kT,Tc,J,F,fricke ] = ising_inputs( input_str );

%

%Run Ising model in a loop, varying F
F = J*linspace( -2, 2, 20 );
F=0;
for n=1:length(F)
    ising_model( show_animation,show_s_vs_iter, N,I,R,x,kT,Tc,J,F(n),fricke );
end

%%

%Ising model
function ising_model( show_animation,show_s_vs_iter, N,I,R,x,kT,Tc,J,F,fricke )

%Fontsize for graphs
fsize = 18;

%Heat capacity plot limit Cmax /eV/K
Cmax = 1e-5;

%Run metropolis algorithm for range of temperatures
disp(' ');
disp(' ISING MODEL OF FERROMAGNETISM by Andy French Oct 2019')
disp(' ');
disp([' Chosen inputs are: N=',num2str(N),' R=',num2str(R),' I=',num2str(I),...
    ' x=',num2str(x(1)),' to ',num2str(x(end)),...
    ' Nx=',num2str(length(x)),' F_by_J=',num2str(F/J)] );
disp(' ');

%Start clock
t0 = tic;

%Initialize stores of mean spin, mean absolute spin, mean energy per spin,
%heat capacity, and standard deviations of these.
Nx = numel(x);
sm = zeros(1,R*Nx) ; xx = zeros(1,R*Nx);
abs_sm = zeros(1,Nx); Em = zeros(1,Nx); C = zeros(1,Nx);
abs_sm_sd = zeros(1,Nx); Em_sd = zeros(1,Nx); C_sd = zeros(1,Nx);

%Run R repeats of Metropolis algorithm for each x = T/Tc value
i = 1;
for n=1:Nx
    Emr = zeros(1,R); Cr = zeros(1,R); abs_smr = zeros(1,R);
    for r=1:R
        %Display message to command window
        disp([' Ising model run: ',num2str(i),'/',num2str( R*length(x) ),...
            ' T/Tc = ',num2str( x(n) ),'. Repeat ',...
            num2str(r),' of ',num2str(R)]);
        
        %Run I*N*N iterations of Meropolis algorithm
        [ s,smi,Emi,Ci ] = metropolis( N, I, kT(n), Tc*x(n), Tc, J, F,...
            show_animation, show_s_vs_iter, fsize, fricke );
        
        %Update output stores
        xx(i) = x(n); sm(i) = smi;
        Emr(r) = Emi; Cr(r) = Ci; abs_smr(r) = abs(smi);
        
        %Update loop counter
        i = i + 1;
    end
    
    %Store averages of absolute spin, mean energy/spin, heat capacity
    %and standard deviations of these
    abs_sm(n) = mean( abs_smr ); abs_sm_sd(n) = std( abs_smr );
    Em(n) = mean( Emr); Em_sd(n) = std( Emr );
    C(n) = mean( Cr); C_sd(n) = std( Cr );
end

%Create output directory
dirname = strrep( ['N=',num2str(N),' R=',num2str(R),' I=',num2str(I),...
    ' x=',num2str(x(1)),' to ',num2str(x(end)),...
    ' Nx=',num2str(length(x)),' F_by_J=',num2str(F/J)],'.','p' );
mkdir(dirname);

%Save data to a .mat file and display completion message
save( [dirname,'\ising'] );
disp(' ');
disp([' Simulation complete in ',num2str(toc(t0)),'s.']);
disp(' ');

%Plot average spin vs T/Tc
figure('color',[1 1 1],'name','Ising model');
plot( xx,  sm, '.' ); hold on;
patch( [x,fliplr(x)],[ abs_sm + abs_sm_sd, fliplr( abs_sm - abs_sm_sd )],'g','facealpha',0.5 );
plot( x, abs_sm, 'r-','linewidth',2 ); axis tight
try xlim( [min(x),max(x)] ); end
xlabel('T/T_c','fontsize',fsize); ylabel('Average spin','fontsize',fsize);
title(['Average spin. J=',num2str(J,2),'eV. T_c=',num2str(Tc),' K'],'fontsize',fsize);
grid on; box on;
set(gca,'fontsize',fsize);
print(gcf,[dirname,'\Ising spin vs T N=',num2str(N),'.png'],'-dpng','-r300');
close(gcf);

%Plot average spin energy /eV vs T/Tc
figure('color',[1 1 1],'name','Ising model'); hold on;
patch( [x,fliplr(x)],[ Em + Em_sd, fliplr( Em - Em_sd )]/J,'g','facealpha',0.5 );
plot( x, Em/J, 'r-','linewidth',2 ); axis tight
try xlim( [min(x),max(x)] ); end
xlabel('T/T_c','fontsize',fsize); ylabel('Spin energy/coupling energy J','fontsize',fsize);
title(['Spin energy/J. J=',num2str(J,2),'eV. T_c=',num2str(Tc),' K'],'fontsize',fsize);
grid on;  box on;
set(gca,'fontsize',fsize);
print(gcf,[dirname,'\Spin energy vs T N=',num2str(N),'.png'],'-dpng','-r300');
close(gcf);

%Plot heat capacity /eV per K vs T/Tc
i = isnan(C); C(i) = []; C_sd(i) = []; x(i) = [];
i = isnan(C_sd); C(i) = []; C_sd(i) = []; x(i) = [];
figure('color',[1 1 1],'name','Ising model'); hold on;
patch( [x,fliplr(x)],[ C + C_sd, fliplr( C - C_sd )],'g','facealpha',0.5 );
plot( x, C, 'r-','linewidth',2 ); axis tight
ylim([-0.5*Cmax,1.5*Cmax]);
try xlim( [min(x),max(x)] ); end
xlabel('T/T_c','fontsize',fsize); ylabel('Heat capacity / eV/K','fontsize',fsize);
title(['Heat capacity. J=',num2str(J,2),'eV. T_c=',num2str(Tc),' K'],'fontsize',fsize);
grid on; box on;
set(gca,'fontsize',fsize);
print(gcf,[dirname,'\Heat capacity vs T N=',num2str(N),'.png'],'-dpng','-r300');
close(gcf);

%%

%Ising model inputs
function [ show_animation,show_s_vs_iter,N,I,R,x,kT,Tc,J,F,fricke ] = ising_inputs( input_choice )
if strcmp( input_choice, 'N=10, I=2000, R=100, x=0.0 to 2.0, Nx=100, F=0' )
    %Show Metropolis algorithm animation (creates a .avi movie)
    show_animation = 0;
    
    %Plots agraph of average spin vs iteration number)
    show_s_vs_iter = 0;
    
    %Number of magnetic dipoles (NxN grid)
    N = 10;
    
    %Iterations of Metropolis algorithm (as a multiple of N^2)
    I = 2000;
    
    %Repeats of Metropolis algorithm per temperature
    R = 100;
    
    %Temperature range where x = T/Tc
    xmin = 0.0; xmax = 2;
    Nx = 100;
    
    %Curie temperature /K
    % https://en.wikipedia.org/wiki/Curie_temperature
    % Iron 1043, Cobalt 1400, Nickel 627
    Tc = 1043;
    
    %Define F = m*B/qe (in eV) in terms of J. Note m = -9284.764e-27 J/T for
    %a single electron. F will be multiplied by J below.
    F = -1;
    
    %Use Fricke vectorized method in Metropolis algorithm. This makes things
    %much faster, but you will need more iterations I.
    fricke = 0;
elseif strcmp( input_choice, 'N=100, I=50, R=1, x=0.5 with animation, F=0' )
    show_animation = 1;
    show_s_vs_iter = 1;
    N = 100;
    I = 50;
    R = 1;
    xmin = 0.5; xmax = 0.5;
    Nx = 1;
    Tc = 1043;
    F = 0;
    fricke = 0;
elseif strcmp( input_choice, 'N=10, I=100, R=10, x=0.0 to 0.4, Nx=3 check convergence, F=0' )
    show_animation = 0;
    show_s_vs_iter = 1;
    N = 10;
    I = 100;
    R = 10;
    xmin = 0.0; xmax = 0.4;
    Nx = 3;
    Tc = 1043;
    F = 0;
    fricke = 0;
elseif strcmp( input_choice, 'N=10, I=25, R=10, x=0.0 to 2.0, Nx=10, F=0' )
    show_animation = 0;
    show_s_vs_iter = 0;
    N = 10;
    I = 25;
    R = 10;
    xmin = 0.0; xmax = 2;
    Nx = 10;
    Tc = 1043;
    F = 0;
    fricke = 1;
elseif strcmp( input_choice, 'Fricke N=100, I=1000, R=100, x=0.0 to 2.0, Nx=100, F=0' )
    show_animation = 0;
    show_s_vs_iter = 0;
    N = 100;
    I = 1000;
    R = 100;
    xmin = 0.0; xmax = 2.0;
    Nx = 100;
    Tc = 1043;
    F = 0;
    fricke = 1;
elseif strcmp( input_choice, 'Fricke N=500, I=1000, R=1, x=0.5 with animation, F=0' )
    show_animation = 1;
    show_s_vs_iter = 1;
    N = 500;
    I = 1000;
    R = 1;
    xmin = 0.5; xmax = 0.5;
    Nx = 1;
    Tc = 1043;
    F = 0;
    fricke = 1;
else
    %Fricke N=100, I=1000, R=1, x=0.5 with animation, F=0
    show_animation = 1;
    show_s_vs_iter = 1;
    N = 100;
    I = 1000;
    R = 1;
    xmin = 1.5; xmax = 1.5;
    Nx = 1;
    Tc = 1043;
    F = 0;
    fricke = 1;
end

%Charge on the electron /C
qe = 1.6e-19;

%Boltzmann's constant (JK^-1);
kB = 1.38e-23;

%Magnetic dipole nearest-neighbour coupling energy /eV
%from Curie temperature (for 2D rectangular grid Ising model)
%using Lars Onsager's classic result
J = Tc * log( 1+sqrt(2) )*kB / (2*qe);
F = F*J;

%Determine array of x = T/Tc
x = linspace( xmin, xmax, Nx );

%Determine kB*T in eV
kT = Tc*x*kB/qe;

%%

%metropolis
% Determine average spin state using Metropolis algorithm
function [ s,sm,Em,C ] = metropolis( N, I, kT, T, Tc, J, F,...
    show_animation,show_s_vs_iter, fsize, fricke )

%Initialize spin states (+1,-1) of grid
s = sign( 0.5 - rand(N,N) );

%Determine mean spin, mean spin energy /eV and heat capacity /eVK^-1
[ sm, Em, C ] = mean_spin_calcs( s, J, F, kT, T );
sm_store = [sm,zeros(1,I-1)];

if show_animation==1
    %Initialize plot
    figure('name','ising','color',[1 1 1],'units','normalized','position',[0,0,1,1] );
    p = pcolor(s); axis off; axis equal;
    tit = title({['iteration = ',num2str(1),'/',num2str(I)],...
        ['Mean spin=',num2str(sm),', T/Tc=',num2str(T/Tc)]},'fontsize',fsize );
    shading flat; box on;
    colormap( [0,0,0;1,1,1] );
    
    %Print first frame
    print( gcf, ['Ising FIRST FRAME ','N=',num2str(N),...
        ', x=',strrep( num2str(T/Tc),'.','p'),'.png'],'-dpng','-r300' );
    
    %Open videowriter object
    vidObj = VideoWriter( ['Ising animation ','N=',num2str(N),...
        ', x=',strrep( num2str(T/Tc),'.','p'),'.avi'] );
    open(vidObj);
end

%Run Ising model via the Metropolis algorithm
% Choose a random magnetic dipole to be flipped and work out total energy
% change. If energy change dE is negative, flip. If energy change is
% positive, flip if exp( -dE/(kB*T) ) > rand
for i=1:I
    if fricke==0
        
        %Original Metropolis method, making one change at a time.
        
        %Determine N^2 randomly sleected array elements of s
        [r,c] = ind2sub( [N,N], randi(N*N,[1,N^2]) );
        
        for j=1:N*N
            %Get rows, cols of randomly selected magnetic dipole
            row = r(j); col = c(j);
            
            %Find its nearest neighbors
            above = mod(row - 1 - 1, size(s,1)) + 1;
            below = mod(row + 1 - 1, size(s,1)) + 1;
            left  = mod(col - 1 - 1, size(s,2)) + 1;
            right = mod(col + 1 - 1, size(s,2)) + 1;
            
            neighbours = [    s(above,col);
                s(row,left);                s(row,right);
                s(below,col)   ];
            
            %Decide whether to change the sign of spin s(n)
            dE = 2*J * sum(neighbours) * s(row,col) + 2*F*s(row,col) ; %in eV
            if ( dE<0 ) || ( exp(-dE/kT) >= rand() )
                s(row,col) = -s(row,col);
            end
        end
    else
        %Fricke's much faster vectorized version of the Metropolis algorithm
        %(Tobin Fricke 2006)
        
        % Calculate the number of neighbours of each cell
        neighbours = circshift( s, [ 0 1]) + ...
            circshift( s, [ 0 -1]) + ...
            circshift( s, [ 1 0]) + ...
            circshift( s, [-1 0]);
        
        % Calculate the change in energy /eV of flipping a spin
        dE = 2*J *( s .* neighbours) + 2*F*s ;
        
        % Decide which transitions will occur
        transitions = ( rand(N) < exp( -dE/kT ) ).*(rand(N) < 0.1) * -2 + 1;
        
        % Flip spins
        s = s.*transitions;
    end
    
    %Determine mean spin, mean spin energy /eV and heat capacity /eVK^-1
    [ sm, Em, C ] = mean_spin_calcs( s, J, F, kT, T );
    sm_store(i)  = sm;
    
    %Update animation, if show_animation==1
    if show_animation ==1
        
        %Update figure
        set( p,'cdata',s );
        set( tit, 'string',{ ['iteration = ',num2str(i),'/',num2str(I)],...
            ['Mean spin=',num2str(sm),', T/Tc=',num2str(T/Tc)] },'fontsize',fsize );
        drawnow;
        
        %Add frame to .avi video
        writeVideo( vidObj, getframe );
    end
end
if show_animation==1
    %Save last frame
    print( gcf, ['Ising LAST FRAME ','N=',num2str(N),...
        ', x=',strrep( num2str(T/Tc),'.','p'),'.png'],'-dpng','-r300' );
    close(gcf);
    %Close videowriter object
    close(vidObj);
end

%Plot progress of average spin vs iteration
if show_s_vs_iter==1
    figure; plot( 1:I,sm_store );
    xlabel('Iteration/N^2','fontsize',18);
    ylabel('Average spin','fontsize',18);
    title(['Mean spin vs iteration. N=',num2str(N),...
        ', T/Tc=',num2str(T/Tc)],'fontsize',fsize);
    set(gca,'fontsize',fsize); grid on;
    print( gcf, ['smean vs iter ','N=',num2str(N),...
        ', x=',strrep( num2str(T/Tc),'.','p'),'.png'],'-dpng','-r300' );
    close(gcf);
end

%%

%Mean energy per spin E /eV given N*N spin array (of -1,1) and coupling
%energy J/eV. Variance of E is also computed, which is then used to
%determine the heat capacity C in eV/K.
function [ sm, Em, C ] = mean_spin_calcs( s, J, F, kT, T )

%Calculate average spin
sm = mean( s(:) );

%Determine spins of nearest neighbour interactions. A neat implementation
%from https://uk.mathworks.com/matlabcentral/fileexchange/...
%62194-ising-model-and-metropolis-algorithm
sumOfNeighbors = ...
    circshift(s, [ 0  1]) + circshift(s, [ 0 -1]) ...
    + circshift(s, [ 1  0]) + circshift(s, [-1  0]);

%Matrix of spin coupling energies
E = - J * s .* sumOfNeighbors - F*s;

%Average energy per spin. Note 0.5 multiplier to avoid double counting
% spin coupling interactions
Em  = 0.5 * mean( E(:) );

%Compute variance of mean energy (which can be used to compute
%specific heat capacity C = Var(E)/(kB*T^2) in eV/K
C = ( 0.25*mean( E(:).^2 ) - Em^2 )/( kT*T );

%End of code




