%ising_plotter
% Loads .mat files 'ising' produced by model runs and combines data in
% surface and conventional graphs.
%
% LAST UPDATED by Andy French October 2019

function ising_plotter

%Interpolated surface grid
Ns = 500;

%Fontsize
fsize = 14;

%Heat capacity plot limits /eV/K
Cmax = 1e-5;
Climits = [-0.5*Cmax,2*Cmax];

%Average spin plot limits
spin_limits = [-1.2,1.2];

%Average spin energy /coupling energy J
Em_limits = [-3.2,0];

%

%Initialize matrices of mean spin, absolute mean spin, heat capacity, mean
%energy, and corresponding standard deviations
abs_sm = []; Em = []; C = [];
abs_sm_sd = []; Em_sd = []; C_sd = [];

%Applied field energy
F = [];

%Look in selected directories of ising.mat
dirnames = uipickfiles;
if isempty(dirnames)
    return
else
    for n=1:length(dirnames)
        %Load file ising.mat and add to structure array I
        I(n) = load( [dirnames{n},'\ising.mat'] );
        
        %Update graphs
        ising_plot_commands( dirnames{n}, Climits, spin_limits, Em_limits  );
        
        %Add data
        F(n) = I(n).F;
    end
end

%Get common title string
ctstr = ['N=',num2str(I(n).N),', R=',num2str(I(n).R),', I=',num2str(I(n).I),...
    ', Tc=',num2str(I(n).Tc),'K, J=',num2str(I(n).J),'eV'] ;

%Sort by F
[F,i] = sort(F);
I = I(i);

%Add data
for n=1:length(F)
    abs_sm = [ abs_sm, I(n).abs_sm.' ];
    Em = [ Em, I(n).Em.' ];
    C = [ C, I(n).C.' ];
    abs_sm_sd = [ abs_sm_sd, I(n).abs_sm_sd.' ];
    Em_sd = [ Em_sd, I(n).Em_sd.' ];
    C_sd = [ C_sd, I(n).C_sd.' ];
end

%Define monotomic plot coordinate vectors and turn into matrices
xi = linspace( I(n).x(1), I(n).x(end), Ns );
Fi = linspace( min(F), max(F), Ns );
[Fi,xi] = meshgrid( Fi,xi );

%Generate X and F matrices
[F,x] = meshgrid( F, I(n).x );

%Interpolate
abs_sm = interp2( F, x, abs_sm, Fi, xi,'spline'  );
abs_sm_sd = interp2( F, x, abs_sm_sd, Fi, xi,'spline' );
Em = interp2( F, x, Em, Fi, xi,'spline'  );
Em_sd = interp2( F, x, Em_sd, Fi, xi,'spline'  );
C = interp2( F, x, C, Fi, xi,'spline'  );
C_sd = interp2( F, x, C_sd, Fi, xi,'spline'  );

%Absolute mean spin
figure; surf( xi, Fi/I(n).J, abs_sm, abs_sm );
shading interp; interp_colormap(Ns); caxis([0,1])
colorbar('fontsize',fsize); view(2);
axis tight; set(gca,'fontsize',fsize );
xlabel( 'T/Tc','fontsize',fsize); ylabel( ' F/J','fontsize',fsize );
title(['Mean abs spin ',ctstr],'fontsize',fsize );
print( gcf, 'Mean abs spin.png','-dpng','-r300' ); clf;

%Absolute mean spin std
surf( xi, Fi/I(n).J, abs_sm_sd, abs_sm_sd );
shading interp; interp_colormap(Ns);
colorbar('fontsize',fsize); view(2);
axis tight; set(gca,'fontsize',fsize );
xlabel( 'T/Tc','fontsize',fsize); ylabel( ' F/J','fontsize',fsize );
title(['Mean abs spin sd ',ctstr],'fontsize',fsize );
print( gcf, 'Mean abs spin sd.png','-dpng','-r300' ); clf;

%Mean energy per spin
surf( xi, Fi/I(n).J, Em/I(n).J, Em/I(n).J);
shading interp; interp_colormap(Ns);
colorbar('fontsize',fsize); view(2);
axis tight; set(gca,'fontsize',fsize );
xlabel( 'T/Tc','fontsize',fsize); ylabel( ' F/J','fontsize',fsize );
title(['Emean(eV)/J(eV) ',ctstr],'fontsize',fsize );
print( gcf, 'Mean energy.png','-dpng','-r300' ); clf;

%Mean energy per spin std
surf( xi, Fi/I(n).J, Em_sd/I(n).J, Em_sd/I(n).J );
shading interp; interp_colormap(Ns);
colorbar('fontsize',fsize); view(2);
axis tight; set(gca,'fontsize',fsize );
xlabel( 'T/Tc','fontsize',fsize); ylabel( ' F/J','fontsize',fsize );
title(['Em/J sd ',ctstr],'fontsize',fsize );
print( gcf, 'Mean energy sd.png','-dpng','-r300' ); clf;

%Heat capacity
surf( xi, Fi/I(n).J, C, C );
zlim([0,Cmax]); caxis([0,Cmax]);
shading interp; interp_colormap(Ns);
colorbar('fontsize',fsize); view(2);
axis tight; set(gca,'fontsize',fsize );
xlabel( 'T/Tc','fontsize',fsize); ylabel( ' F/J','fontsize',fsize );
title(['Heat capacity eV/K ',ctstr],'fontsize',fsize );
print( gcf, 'Heat capacity.png','-dpng','-r300' ); clf;

%Heat capacity std
surf( xi, Fi/I(n).J, C_sd, C_sd );
zlim([0,Cmax]); caxis([0,Cmax]);
shading interp; interp_colormap(Ns);
colorbar('fontsize',fsize); view(2);
axis tight; set(gca,'fontsize',fsize );
xlabel( 'T/Tc','fontsize',fsize); ylabel( ' F/J','fontsize',fsize );
title(['Heat capacity eV/K sd ',ctstr],'fontsize',fsize );
print( gcf, 'Heat capacity sd.png','-dpng','-r300' ); close(gcf);

%%

%ising_plot_commands
function ising_plot_commands( ising_dirname, Climits, spin_limits, Em_limits )
load( [ising_dirname,'\ising.mat'] );
dirname = ising_dirname;

%Plot average spin vs T/Tc
figure('color',[1 1 1],'name','Ising model');
plot( xx,  sm, '.' ); hold on;
patch( [x,fliplr(x)],[ abs_sm + abs_sm_sd, fliplr( abs_sm - abs_sm_sd )],'g','facealpha',0.5 );
plot( x, abs_sm, 'r-','linewidth',2 ); axis tight
try xlim( [min(x),max(x)] ); end; ylim(spin_limits)
xlabel('T/T_c','fontsize',fsize); ylabel('Average spin','fontsize',fsize);
title(['Average spin. J=',num2str(J,2),'eV. T_c=',num2str(Tc),' K'],'fontsize',fsize);
grid on; box on;
set(gca,'fontsize',fsize);
print(gcf,[dirname,'\Ising spin vs T N=',num2str(N),'.png'],'-dpng','-r300');
close(gcf);

%Plot average spin energy /eV vs T/Tc
figure('color',[1 1 1],'name','Ising model'); hold on;
patch( [x,fliplr(x)],[ Em + Em_sd, fliplr( Em - Em_sd )]/J,'g','facealpha',0.5 );
plot( x, Em/J, 'r-','linewidth',2 ); axis tight
try xlim( [min(x),max(x)] ); end; ylim(Em_limits);
xlabel('T/T_c','fontsize',fsize); ylabel('Spin energy/coupling energy J','fontsize',fsize);
title(['Spin energy/J. J=',num2str(J,2),'eV. T_c=',num2str(Tc),' K'],'fontsize',fsize);
grid on;  box on;
set(gca,'fontsize',fsize);
print(gcf,[dirname,'\Spin energy vs T N=',num2str(N),'.png'],'-dpng','-r300');
close(gcf);

%Plot heat capacity /eV per K vs T/Tc
i = isnan(C); C(i) = []; C_sd(i) = []; x(i) = [];
i = isnan(C_sd); C(i) = []; C_sd(i) = []; x(i) = [];
figure('color',[1 1 1],'name','Ising model'); hold on;
patch( [x,fliplr(x)],[ C + C_sd, fliplr( C - C_sd )],'g','facealpha',0.5 );
plot( x, C, 'r-','linewidth',2 ); axis tight; ylim(Climits);
try xlim( [min(x),max(x)] ); end
xlabel('T/T_c','fontsize',fsize); ylabel('Heat capacity / eV/K','fontsize',fsize);
title(['Heat capacity. J=',num2str(J,2),'eV. T_c=',num2str(Tc),' K'],'fontsize',fsize);
grid on; box on;
set(gca,'fontsize',fsize);
print(gcf,[dirname,'\Heat capacity vs T N=',num2str(N),'.png'],'-dpng','-r300');
close(gcf);

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code